@@ -12,6 +12,8 @@
 Functions to be able to use the old SetupProject (CMT) from within
 LbEnv.ProjectEnv.
 '''
+from __future__ import print_function
+
 __author__ = 'Marco Clemencic <marco.clemencic@cern.ch>'
 
 import os
@@ -329,7 +331,8 @@ def _extract_version(project, version):
 def FindProjectVersions(project,
                         search_path,
                         user_area=None,
-                        ignore_not_ready=False):
+                        ignore_not_ready=False,
+                        requested_version=None):
     """Given a project name, discovers all the matching project/versions using
     the provided user_area and the the search_path (a list of directories).
     Returns a list of tuples with (name, version, realname, basepath) where
@@ -366,7 +369,26 @@ def FindProjectVersions(project,
     # find all project directories in the searchpath
     PROJECT = project.upper()
     # search_path = os.environ["CMTPROJECTPATH"].split(os.path.pathsep)
+
+    # special cases for the version, if explicitly requested
+    special_project_dirs = []
+    if requested_version:
+        # user project or nightlies like version
+        special_project_dirs.append('%s_%s' % (project, requested_version))
+
+        prj = fixProjectCase(project)
+        if prj != project:
+            # same as above, but with canonical project name (if applicable)
+            special_project_dirs.append('%s_%s' % (prj, requested_version))
+        
+        # release style project name+version
+        special_project_dirs.append(os.path.join(PROJECT, '%s_%s' % (PROJECT, requested_version)))
+
     for d in search_path:
+        versions.extend(
+            (project, requested_version, v, d) for v in special_project_dirs
+            if os.path.isdir(os.path.join(d, v))
+        )
         p = os.path.join(d, PROJECT)
         if os.path.isdir(p):
             if isProject(p, ignore_not_ready, cvmfs=True):
@@ -623,7 +645,7 @@ def cmt(command, args=[], environment=None, cwd=None):
         environment = os.environ
     return Popen(
         cmd, shell=True, env=environment, cwd=cwd, stdout=PIPE,
-        stderr=STDOUT).communicate()[0]
+        stderr=STDOUT).communicate()[0].decode()
 
 
 class GetNightlyCMTPROJECTPATH(ContentHandler):
@@ -633,7 +655,7 @@ class GetNightlyCMTPROJECTPATH(ContentHandler):
     from xml.sax import parse
     getter = GetNightlyCMTPROJECTPATH("lhcb2","Mon")
     parse(filename, getter)
-    print getter.value()
+    print(getter.value())
     """
 
     def __init__(self, slot, day=None):
@@ -985,14 +1007,11 @@ class SetupProject(object):
         Check that the current environment is usable.
         """
         self._debug("----- check_environment() -----")
-        output = self.cmt("version")
-        if output.find('command not found') > 0 or output.find(
-                'not recognized') > 0:
-            self._error("I cannot find cmt (it is not in your PATH)")
-            return False
         if 'BINARY_TAG' in self.environment:
-            if 'CMTCONFIG' in self.environment:
-                self._warning('both CMTCONFIG and BINARY_TAG set, using BINARY_TAG')
+            if ('CMTCONFIG' in self.environment and
+                    self.environment['CMTCONFIG'] != os.environ['BINARY_TAG']):
+                self._warning(
+                    'both CMTCONFIG and BINARY_TAG set, using BINARY_TAG')
             self.environment['CMTCONFIG'] = os.environ['BINARY_TAG']
         if not 'CMTCONFIG' in self.environment:
             self._error("You do not have CMTCONFIG set")
@@ -1005,6 +1024,33 @@ class SetupProject(object):
             # do nothing
             # FIXME: is this an error?
             return
+        # make sure we have CMT in the path
+        from LbEnv import which
+        if not which('cmt'):
+            self._debug('"cmt" not in the default environemnt, '
+                        'I look for it...')
+            import LbPlatformUtils
+            from LbEnv.Bootstrap import bin_path
+            cmt_cmd = which(
+                'cmt',
+                path=os.pathsep.join(
+                    # this is a convoluted way of deducing the LbEnv standard
+                    # PATH from CMAKE_PREFIX_PATH
+                    bin_path([
+                        os.path.dirname(d)
+                        for d in self.search_path if d.endswith('lhcb')
+                    ],
+                             host_os=LbPlatformUtils.host_os(),
+                             host_flavour='{0}-{4}'.format(*os.uname()))))
+            if cmt_cmd:
+                self._debug('using %s', cmt_cmd)
+                self.environment['PATH'] = os.pathsep.join(
+                    [self.environment['PATH'],
+                     os.path.dirname(cmt_cmd)])
+            else:
+                self._error("I cannot find cmt")
+                raise RuntimeError("Environment sanity check failed")
+
         if self.project_info.policy == 'old':  # old style project
             self._debug("----- old style project -----")
             if 'CMTPROJECTPATH' in self.environment:
@@ -1025,9 +1071,9 @@ class SetupProject(object):
                     self.environment['CMTPATH'] = ep_pi.project_dir
             # get all the variables from <Project>Env
             localEnv = dict(self.environment)
-            ShellParser[self.shell](
-                self.cmt("setup", "-" + self.shell,
-                         self.project_info._projectenv_cmt_dir), localEnv)
+            ShellParser[self.shell](self.cmt(
+                "setup", "-" + self.shell,
+                self.project_info._projectenv_cmt_dir), localEnv)
             # get the CMTPATH from <Project>Env without variable expansion
             cmtpath = os.popen("cmt show set CMTPATH").readlines()[-1].strip()
             if cmtpath.startswith("CMTPATH="):  # remove head of the line
@@ -1069,13 +1115,14 @@ class SetupProject(object):
         elif self.opts.mktemp:
             fd, outname = mkstemp()
             self.output_file = os.fdopen(fd, "w")
-            print outname
+            print(outname)
             self.opts.mktemp = None  # reset the option value to avoid to reuse it
             close_output = True
         # write the data
         self.output_file.write(data)
         self.output_file.write("\n")  # @TODO: this may be avoided
-        if close_output: self.output_file.close()
+        if close_output:
+            self.output_file.close()
 
     def _prepare_parser(self):
         from optparse import OptionParser, \
@@ -1269,6 +1316,32 @@ class SetupProject(object):
             help="Use the specified path as User_release_area instead of "
             "the value of the environment variable.")
 
+        def siteroot_option(option, opt_str, value, parser):
+            from LbEnv.Bootstrap import collect_roots, search_path
+            parser.values.dev_dirs.extend(search_path(collect_roots(value)))
+            self.environment['SITEROOT'] = value  # required by old LCGCMT
+
+        parser.add_option(
+            '-r',
+            '--siteroot',
+            type='string',
+            action='callback',
+            callback=siteroot_option,
+            help='path to the installation root, used to add default search '
+            'path')
+
+        def unset_option(option, opt_str, value, parser):
+            if value in self.environment:
+                del self.environment[value]
+
+        parser.add_option(
+            '-u',
+            '--unset',
+            type='string',
+            action='callback',
+            callback=unset_option,
+            help='unset an variable before preparing the environment')
+
         def external_version_option(_option, opt_str, value, parser):
             if len(parser.largs) < 2:
                 raise OptionValueError(
@@ -1409,18 +1482,36 @@ class SetupProject(object):
             '(must be specified before --nightly)')
 
         def nightly_option(_option, opt_str, _value, _parser):
+            valid_value = re.compile(
+                r'^(mon|tue|wed|thu|fri|sat|sun|today|yesterday|latest|'
+                r'\d{4}-\d\d-\d\d|\d+)$', re.IGNORECASE)
+            day = 'Today'
+
             parser.values.dev = True
-            if len(parser.rargs) < 1:
+            rargs = parser.rargs
+
+            try:
+                slot = rargs.pop(0)
+            except IndexError:
                 raise OptionValueError(
-                    "%s must be followed by the slot of the nightlies and optionally by the day"
-                    % opt_str)
-            slot = parser.rargs.pop(0)
-            if len(parser.rargs) and parser.rargs[0].capitalize() in days:
-                day = parser.rargs.pop(0).capitalize()
-            else:
-                day = days[time.localtime()[6]]
-                # In Python 2.3
-                # day = days[datetime.date.today().weekday()]
+                    '%s must be followed by the slot of the '
+                    'nightlies and optionally by the build id' % opt_str)
+
+            if '/' in slot:
+                slot, day = slot.split('/', 1)
+                if valid_value.match(day):
+                    day = day.capitalize()
+            elif rargs:
+                match = valid_value.match(rargs[0])
+                if match:
+                    day = rargs.pop(0).capitalize()
+                    import logging
+                    logging.warning(
+                        'deprecated slot id specification: use "... '
+                        '--nightly %s/%s ..." instead', slot, day)
+            if day == 'Latest':  # special case
+                day = 'latest'
+
             # Locate the requested slot in the know nightlies directories
             global nightly_bases
             nightly_bases += [
@@ -1454,9 +1545,8 @@ class SetupProject(object):
                 data = {}
                 exec open(confSumm_file).read() in data  #IGNORE:W0122
                 # Get the list and convert it to strings
-                extraCMTPROJECTPATH = filter(str,
-                                             data.get("cmtProjectPathList",
-                                                      []))
+                extraCMTPROJECTPATH = filter(
+                    str, data.get("cmtProjectPathList", []))
             elif os.path.exists(config_file):  # Try with the XML configuration
                 extraCMTPROJECTPATH = getNightlyCMTPROJECTPATH(
                     config_file, slot, day)
@@ -1629,6 +1719,18 @@ class SetupProject(object):
                 hat, pack, ver = m.groups()
                 if ver is None:
                     ver = 'v*'
+                if hat is None:
+                    from LbEnv import getPackageNames
+                    suffix = '/' + pack
+                    hat = ''
+                    for p in getPackageNames():
+                        if p.endswith(suffix):
+                            hat = p.split('/', 1)[0]
+                            break
+                    if hat:
+                        self._warning(
+                            'wrong data package name used (%s), the'
+                            ' correct name is %s/%s', pack, hat, pack)
                 req.write("use %s %s %s\n" % (pack, ver, hat))
             else:  # for any other case, just use the provided string
                 req.write("use %s\n" % u)
@@ -1656,8 +1758,8 @@ class SetupProject(object):
         #  some packages may change the default version, but the command line
         #  must win)
         for e in self.ext_versions:
-            req.write('macro %s_config_version "%s"\n' %
-                      (e, self.ext_versions[e]))
+            req.write(
+                'macro %s_config_version "%s"\n' % (e, self.ext_versions[e]))
 
         # use external location relative to the LCG Interfaces for the local installation
 
@@ -1791,7 +1893,8 @@ class SetupProject(object):
 
     def _prepend_prompt(self):
         new_prompt = ""
-        if os.environ["SP_PROMPT"] == "without_version" or not self.project_info.version:
+        if os.environ[
+                "SP_PROMPT"] == "without_version" or not self.project_info.version:
             if self.shell == "sh":
                 new_prompt = 'if [[ -n "$PS1" ]]; then\n'
                 # assume that ZSH_VERSION is always set in zsh and never in bash
@@ -1920,7 +2023,7 @@ class SetupProject(object):
         if self.dev_dirs:
             self.search_path += self.dev_dirs
         # default locations
-        projpathvars = ["LHCBPROJECTPATH"]
+        projpathvars = ["CMAKE_PREFIX_PATH", "LHCBPROJECTPATH"]
         if self.keep_CMTPROJECTPATH:
             projpathvars.insert(0, "CMTPROJECTPATH")
         for v in projpathvars:
@@ -1940,7 +2043,8 @@ class SetupProject(object):
                          (" with user area in '%s'" % self.user_area)) or ""))
         versions = FindProjectVersions(self.project_name, self.search_path,
                                        self.user_area,
-                                       self.opts.ignore_not_ready)
+                                       self.opts.ignore_not_ready,
+                                       self.project_version)
 
         if not versions:
             self._error("Cannot find project '%s'" % self.project_name)
@@ -1952,9 +2056,8 @@ class SetupProject(object):
                     self.project_version = v[1]
                     break
             else:
-                self._error(
-                    "Cannot find version '%s' of %s. Try with --list-versions."
-                    % (self.project_version, self.project_name))
+                self._error("Cannot find version '%s' of %s. Try with --list."
+                            % (self.project_version, self.project_name))
                 return 1
 
         # Now I can print the version if I was asked to do it
@@ -1967,9 +2070,9 @@ class SetupProject(object):
             # version of ROOT
             self._debug('Looking for the LCGCMT version providing ROOT %s' %
                         required_root_version)
-            lcg_version = FindLCGForROOT(required_root_version,
-                                         SortVersionTuples(
-                                             versions, reverse=True))
+            lcg_version = FindLCGForROOT(
+                required_root_version, SortVersionTuples(
+                    versions, reverse=True))
             if lcg_version:
                 # we need the version in the args because it's processed later
                 self.args.insert(0, lcg_version)
@@ -2087,8 +2190,8 @@ class SetupProject(object):
                 self._error("Project %s %s is not available for platform %s" %
                             (p.name, p.version, platform))
                 return 1
-            self._verbose("Project %s %s uses %s policy" % (p.name, p.version,
-                                                            p.policy))
+            self._verbose(
+                "Project %s %s uses %s policy" % (p.name, p.version, p.policy))
 
         if self.opts.platform:
             self.environment['CMTCONFIG'] = self.opts.platform
@@ -2112,6 +2215,13 @@ class SetupProject(object):
         # Process commmand line options
         self.parse_args(args=args)
 
+        # Required for some old LCGCMT
+        if ('SITEROOT' not in self.environment
+                and 'MYSITEROOT' in self.environment):
+            self.environment['SITEROOT'] = self.environment['MYSITEROOT']
+        # this means not-AFS (the only option at the moment)
+        self.environment['CMTSITE'] = 'LOCAL'
+
         # Prepare the profiler instance
         profiler = None
         if self.opts.profile:
@@ -2168,7 +2278,7 @@ class SetupProject(object):
                 self._always(
                     "WARNING - ignoring warnings and errors from CMT:")
                 self._always("\n".join(err))
-        except SetupProjectError, x:
+        except SetupProjectError as x:
             self._error(str(x))
             if 'package CASTOR' in str(x):
                 self._error("Tip: try adding --disable-CASTOR to the options")
@@ -2184,7 +2294,8 @@ class SetupProject(object):
         else:
             messages.append("Using CMTPATH = '%s'" % CMTPATH)
         tmps = self.project_info.name
-        if self.project_info.version: tmps += " %s" % self.project_info.version
+        if self.project_info.version:
+            tmps += " %s" % self.project_info.version
         messages.append('Environment for %s ready.' % tmps)
 
         # collect project infos descriptions
@@ -2275,6 +2386,9 @@ class Comp(SetupProject):
             dest="outshell",
             help="Print the environment as Python "
             "dictionary.")
+        parser.add_option('--container', help='ignored')
+        parser.add_option('--allow-containers', action="store_true", help='ignored')
+        parser.add_option('--prefer-container', action="store_true", help='ignored')
         return parser
 
     def _write_script(self, data):
