@@ -8,12 +8,15 @@
 # granted to it by virtue of its status as an Intergovernmental Organization  #
 # or submit itself to any jurisdiction.                                       #
 ###############################################################################
+from __future__ import print_function
+
 __author__ = 'Marco Clemencic <marco.clemencic@cern.ch>'
 
 import os
 import sys
 import shutil
 from tempfile import mkdtemp
+from nose import SkipTest
 
 
 def setup():
@@ -183,7 +186,7 @@ def _test_compatibility():
     try:
         env = getOldEnvironment(['NotAProject', 'v1r0'])
         assert False, 'exception expected'
-    except SystemExit, x:
+    except SystemExit as x:
         assert x.code == 1
     finally:
         os.environ.clear()
@@ -238,10 +241,27 @@ def test_options_addPlatform():
     opts, _ = parse_args([])
     assert opts.platform == os.environ['CMTCONFIG']
 
-    # prefer BINARY_TAG
-    os.environ['BINARY_TAG'] = 'another'
+    # take BINARY_TAG
+    if 'BINARY_TAG' not in os.environ:
+        os.environ['BINARY_TAG'] = 'default'
+    if 'CMTCONFIG' in os.environ:
+        del os.environ['CMTCONFIG']
     opts, _ = parse_args([])
     assert opts.platform == os.environ['BINARY_TAG']
+
+    # allow equal BINARY_TAG and CMTCONFIG
+    os.environ['BINARY_TAG'] = os.environ['CMTCONFIG'] = 'default'
+    opts, _ = parse_args([])
+    assert opts.platform == os.environ['BINARY_TAG']
+
+    # fail for inconsistent
+    os.environ['BINARY_TAG'] = 'another'
+    try:
+        opts, _ = parse_args([])
+        assert False, 'failure expected'
+    except SystemExit:
+        pass
+    del os.environ['CMTCONFIG']
     del os.environ['BINARY_TAG']
 
     # pretend we are on an unsupported platform
@@ -256,7 +276,6 @@ def test_options_addPlatform():
 
     try:
         # no environment, something should be selected
-        del os.environ['CMTCONFIG']
         LbEnv.defaultPlatform = force_known
         opts, _ = parse_args([])
         assert opts.platform == 'known'
@@ -411,7 +430,7 @@ def test_lookup():
         try:
             findProject('AnotherProject', 'v1r0', os.environ['CMTCONFIG'])
             assert False, 'exception expected'
-        except MissingProjectError, x:
+        except MissingProjectError as x:
             assert str(x)
 
         # test with user area
@@ -431,7 +450,7 @@ def test_lookup():
         try:
             findProject('MyAnalysis', 'v1r0', os.environ['CMTCONFIG'])
             assert False, 'exception expected'
-        except MissingProjectError, x:
+        except MissingProjectError as x:
             assert str(x)
 
         pkg = findDataPackage('MyPackage', 'v2r0')
@@ -449,7 +468,7 @@ def test_lookup():
         try:
             findDataPackage('NoPackage', '*')
             assert False, 'exception expected'
-        except MissingDataPackageError, x:
+        except MissingDataPackageError as x:
             assert str(x)
 
         xml_path = getEnvXmlPath('DerivedProject', 'v1r3',
@@ -466,7 +485,7 @@ def test_lookup():
             xml_path = getEnvXmlPath('OldProject', 'v9r0',
                                      os.environ['CMTCONFIG'])
             assert False, 'exception expected'
-        except MissingManifestError, x:
+        except MissingManifestError as x:
             assert str(x)
 
     path[:] = bk_path
@@ -495,8 +514,8 @@ def test_listVersions():
         expected = [('v1r2', join(tmp, 'MYPROJECT', 'MYPROJECT_v1r2')),
                     ('v1r0', join(tmp, 'MYPROJECT', 'MYPROJECT_v1r0'))]
         observed = list(listVersions('MyProject', os.environ['CMTCONFIG']))
-        print 'expected', expected
-        print 'observed', observed
+        print('expected', expected)
+        print('observed', observed)
         assert expected == observed
 
     path[:] = bk_path
@@ -512,7 +531,7 @@ def _test_profiling():
         profiling.run('sys.exit()')
         assert False, 'exception expected'
     except SystemExit:
-        print sys.stderr.getvalue()
+        print(sys.stderr.getvalue())
     finally:
         sys.stderr = sys.__stderr__
 
@@ -528,7 +547,7 @@ def _test_profiling():
         profiling.run('sys.exit()')
         assert False, 'exception expected'
     except SystemExit:
-        print sys.stderr.getvalue()
+        print(sys.stderr.getvalue())
     finally:
         sys.stderr = sys.__stderr__
 
@@ -540,7 +559,7 @@ def _test_profiling():
         profiling.run('sys.exit()')
         assert False, 'exception expected'
     except SystemExit:
-        print sys.stderr.getvalue()
+        print(sys.stderr.getvalue())
     finally:
         sys.stderr = sys.__stderr__
 
@@ -599,3 +618,22 @@ def test_SearchPath():
 
     assert not SearchPath([])
     assert SearchPath(['a'])
+
+
+def test_incompatibility_report():
+    from LbPlatformUtils.inspect import os_id
+    if not os_id().endswith('centos7'):
+        raise SkipTest()  # only centos7 is incompatible with slc5
+    from subprocess import check_output, CalledProcessError, STDOUT
+    try:
+        check_output([
+            'lb-run', '--platform', 'x86_64-slc5-gcc43-opt', 'LHCb/v31r6p1',
+            'true'
+        ],
+                     stderr=STDOUT)
+        assert False, 'script failure expected'
+    except CalledProcessError as err:
+        print(err.output)
+        import re
+        assert re.search(r'dirac_platform: [^-]+-{}'.format(os_id()),
+                         err.output)
